import json
from ast import literal_eval

from pystarlark.starlark import ffi, lib

__version__ = "0.0.2"


class Starlark:
    def __init__(self):
        self._id = lib.NewThread()

    def exec(self, code):
        if isinstance(code, str):
            code = code.encode()
        lib.ExecFile(self._id, code)

    def eval(self, statement, _raw=False):
        if isinstance(statement, str):
            statement = statement.encode()

        output = lib.Eval(self._id, statement)
        response = ffi.string(output)
        lib.FreeCString(output)
        if _raw:
            return response
        value = json.loads(response)["value"]
        return literal_eval(value)

    def __del__(self):
        lib.DestroyThread(self._id)



from pystarlark import Starlark


def test_fibonacci():
    s = Starlark()
    fibonacci = """
def fibonacci(n=10):
    res = list(range(n))
    for i in res[2:]:
        res[i] = res[i-2] + res[i-1]
    return res
"""
    s.exec(fibonacci)
    assert s.eval("fibonacci(5)") == [0, 1, 1, 2, 3]
    assert s.eval("fibonacci(10)") == [0, 1, 1, 2, 3, 5, 8, 13, 21, 34]



import re
import setuptools

with open("pystarlark/__init__.py", "r", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pystarlark",
    version=version,
    author="Kevin Chung",
    author_email="kchung@nyu.edu",
    description="Python bindings for Starlark in Go",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ColdHeat/pystarlark",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    # I'm not sure what this value is supposed to be
    build_golang={"root": "github.com/ColdHeat/pystarlark"},
    ext_modules=[setuptools.Extension("pystarlark/starlark", ["starlark.go"])],
    setup_requires=["setuptools-golang==2.3.0", "cffi==1.14.3"],
    install_requires=["cffi==1.14.3"],
)
